import './spinner.css'
import { MovieWall } from './objects/movie-wall'
import { HamburgerMenu } from './objects/HamburgerMenu'
import { useState, useEffect } from 'react'
import tmdb from './objects/tmdb'
import { Sidebar } from './objects/sidebar'
import { Movie } from './objects/movie-wall'
import CrossfadeImage from 'react-crossfade-image'
import { WatchProviders } from './objects/whereToWatch'
import useWindowDimensions from './hooks/useWindowDimensions'

export type Config = {
  language: string,
  region: string,
  page: number,
  locale: string
}
class MovieClass implements Movie {
  id = 0
  poster_path = null
  backdrop_path = null
  title = null
  overview = null
  vote_average = 0
}

class WatchProvidersClass implements WatchProviders {
  flatrate = [];
  rent = [];
  buy = [];
  link = "";
}


function App() {
  const [config, setConfig] = useState<Config>({
    language: "es",
    region: "spain",
    page: 1,
    locale: "ES"
  })
  const [movies, setMovies] = useState([new MovieClass()])
  const [backdrop, setBackdrop] = useState("")
  const [chosenMovie, setChosenMovie] = useState(new MovieClass())
  const [similarMoviesAvailable, setSimilarMoviesAvailable] = useState(true)
  const [watchProviders, setWatchProviders] = useState(new WatchProvidersClass())
  useEffect(() => { tmdb.getPopular(config, setMoviesHandler) }, [])
  useEffect(() => {
    const bgString = chosenMovie.backdrop_path ?
      tmdb.makeBgImgUrl(chosenMovie.backdrop_path)
      : tmdb.makeBgImgUrl(movies[0]?.backdrop_path ?? "")
    setBackdrop(bgString)
  }, [movies])
  useEffect(() => {
    tmdb.getWhereToWatch(chosenMovie, setWatchProviders, config)
  }, [chosenMovie])
  //Whenever config changes, refresh data for current selection
  useEffect(() => {
    console.log("CONFIG CHANGED!!")
    tmdb.getMovie(config, chosenMovie.id, setChosenMovie)
    tmdb.getSimilar(config, chosenMovie, setMoviesHandler, setSimilarMoviesAvailable)
  }, [config])
  const crossfadeImageStyles = {
    width: "2560px",
    minHeight: "100%",
    aspectRatio: "16/9"
  }

  const { width } = useWindowDimensions()

  function setMoviesHandler(movies: Movie[]) {
    setMovies(trimMoviesForScreen(movies))
  }

  function trimMoviesForScreen(movies: Movie[]) {
    let desiredLength = 0
    if (width > 640) {
      desiredLength = 18
    } else { desiredLength = 12 }
    //ensure movies.length is a multiple of 6 (so it makes an orphanless grid)
    while (movies.length < desiredLength && movies.length >= 6) { desiredLength = desiredLength - 6 }
    return movies.slice(0, desiredLength)
  }

  return (
    <>
      <div id="background" className="-z-10 absolute">
        <CrossfadeImage src={backdrop} style={crossfadeImageStyles} />
      </div>
      <HamburgerMenu setConfig={setConfig} />
      <main className="flex justify-center items-center gap-8">
        WIDTH: {width}
        <Sidebar className="flex flex-col max-w-sm" movie={chosenMovie} similarMoviesAvailable={similarMoviesAvailable} watchProviders={watchProviders} config={config} />
        <MovieWall
          className="flex flex-wrap gap-2 max-w-5xl justify-center"
          movies={movies}
          setMovies={setMoviesHandler}
          config={config}
          setChosenMovie={setChosenMovie}
          setSimilarMoviesAvailable={setSimilarMoviesAvailable}
        />
      </main>
    </>
  )
}

export default App
